// -----------------------------------------------------------------------
// Litmus - DKFZ SIDT unit testing library
//
// Copyright (c) German Cancer Research Center (DKFZ),
// Software development for Integrated Diagnostic and Therapy (SIDT).
// ALL RIGHTS RESERVED.
// See litmusCopyright.txt or
// http://www.dkfz.de/en/sidt/projects/litmus/copyright.html
//
// This software is distributed WITHOUT ANY WARRANTY; without even
// the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the above copyright notices for more information.
//
//------------------------------------------------------------------------
/*!
// @file
// @version $Revision: 4433 $ (last changed revision)
// @date    $Date: 2013-01-22 14:38:20 +0100 (Di, 22 Jan 2013) $ (last change date)
// @author  $Author: floca $ (last changed by)
*/


#ifndef __LIT_TEST_IMAGE_IO_TPP
#define __LIT_TEST_IMAGE_IO_TPP

#include "litTestImageIO.h"

#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"

#include "itkCastImageFilter.h"

namespace lit {

    template <typename TExternalPixelType, typename TInternalImageType>
     typename TestImageIO<TExternalPixelType,TInternalImageType>::InternalImagePointer
     TestImageIO<TExternalPixelType,TInternalImageType>::
     readImage(const StringType& fileName)
     {
	    typedef itk::ImageFileReader< ExternalImageType  > ImageReaderType;
	    typedef itk::CastImageFilter< ExternalImageType, InternalImageType > CastFilterType;              

	    typename CastFilterType::Pointer  spImageCaster =  CastFilterType::New();
	    typename ImageReaderType::Pointer spImageReader  = ImageReaderType::New();

      spImageReader->SetFileName(fileName.c_str());
	    spImageCaster->SetInput(spImageReader->GetOutput());

      InternalImagePointer spImage = spImageCaster->GetOutput();

	    spImageCaster->Update();

      return spImage;
     };

    template <typename TExternalPixelType, typename TInternalImageType>
     void
     TestImageIO<TExternalPixelType,TInternalImageType>::
     writeImage(const InternalImageType* pImage, const StringType& fileName)
     {
       assert(pImage);
        typedef itk::ImageFileWriter< ExternalImageType  > ImageWriterType;
        typedef itk::CastImageFilter< InternalImageType, ExternalImageType > CastFilterType;              

        typename CastFilterType::Pointer  spImageCaster =  CastFilterType::New();
        typename ImageWriterType::Pointer  spImageWriter  = ImageWriterType::New();

       spImageCaster->SetInput(pImage);
       spImageWriter->SetFileName(fileName.c_str());
       spImageWriter->SetInput(spImageCaster->GetOutput());
       spImageWriter->Update();
     };

}

#endif